package com.qizhen.healsphere.repository.postgresql;

import com.qizhen.healsphere.repository.neo4j.BaseNodeRepository;
import com.qizhen.healsphere.repository.neo4j.entity.BaseEntity;
import com.qizhen.healsphere.repository.neo4j.entity.BaseRelationship;
import com.qizhen.healsphere.repository.neo4j.entity.Edge;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Repository
public class EdgeRepository {

    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Autowired
    private NodeRepository nodeRepository;

    private final RowMapper<Map<String, Object>> edgeMapper = (rs, rowNum) -> {
        Map<String, Object> map = new HashMap<>();
        map.put("id", rs.getLong("id"));
        map.put("category", rs.getString("category"));
        map.put("src_id", rs.getLong("src_id"));
        map.put("dest_id", rs.getLong("dest_id"));
        map.put("name", rs.getString("name"));
        map.put("version", rs.getString("version"));
        map.put("status", rs.getInt("status"));
        return map;
    };

    /**
     * 检查关系是否存在
     */
    public Boolean isExistRelationship(String startLabel, Long startId, String endLabel, Long endId, String relationshipType) {
        String sql = "SELECT COUNT(*) FROM kg_edges WHERE src_id = ? AND dest_id = ? AND category = ? AND status != -1";
        int count = jdbcTemplate.queryForObject(sql, Integer.class, startId, endId, relationshipType);
        return count > 0;
    }

    /**
     * 创建关系
     */
    public void createRelationship(String startLabel, Long startId, String endLabel, Long endId, String relationshipType, Map<String, Object> properties) {
        String sql = "INSERT INTO kg_edges (category, src_id, dest_id, name, status) VALUES (?, ?, ?, ?, 0)";
        jdbcTemplate.update(
            sql,
            relationshipType,
            startId,
            endId,
            relationshipType
        );
    }

    /**
     * 更新关系
     */
    public void updateRelationship(String oldRelationshipType, String newRelationshipType) {
        String sql = "UPDATE kg_edges SET category = ?,name = ? WHERE category = ? AND status != -1";
        jdbcTemplate.update(sql, newRelationshipType, newRelationshipType, oldRelationshipType);
    }

    /**
     * 根据关系名字模糊查询关系类型
     */
    public List<String> findRelationshipNames(String name) {
        String sql = "SELECT DISTINCT category FROM kg_edges WHERE category LIKE ? AND status != -1";
        return jdbcTemplate.queryForList(sql, String.class, "%" + name + "%");
    }

    /**
     * 删除关系
     */
    public void deleteRelationship(String startLabel, Long startId, String endLabel, Long endId, String relationshipType) {
        String sql = "UPDATE kg_edges SET status = -1 WHERE src_id = ? AND dest_id = ? AND category = ?";
        jdbcTemplate.update(sql, startId, endId, relationshipType);
    }

    /**
     * 获取三元组关系
     */
    public List<Edge> triples(String startLabel, String relationship, String endLabel) {
        String sql = "SELECT e.*, n1.category as start_category, n2.category as end_category " +
                    "FROM kg_edges e " +
                    "JOIN kg_nodes n1 ON e.src_id = n1.id " +
                    "JOIN kg_nodes n2 ON e.dest_id = n2.id " +
                    "WHERE n1.category = ? AND n2.category = ? AND e.category = ? " +
                    "AND e.status != -1 AND n1.status != -1 AND n2.status != -1";

        return jdbcTemplate.query(sql, (rs, rowNum) -> {
            Edge edge = new Edge();
            
            // 设置起始节点
            Map<String, Object> startNode = nodeRepository.findNodeById(rs.getLong("src_id"));
            edge.setStartNode(buildBaseEntity(startNode));
            
            // 设置结束节点
            Map<String, Object> endNode = nodeRepository.findNodeById(rs.getLong("dest_id"));
            edge.setEndNode(buildBaseEntity(endNode));
            
            // 设置关系
            BaseRelationship baseRelationship = new BaseRelationship();
            baseRelationship.setId(rs.getLong("id"));
            baseRelationship.setType(rs.getString("category"));
            
            Map<String, Object> properties = new HashMap<>();
            properties.put("name", rs.getString("name"));
            properties.put("version", rs.getString("version"));
            properties.put("status", rs.getInt("status"));
            baseRelationship.setProperties(properties);
            
            edge.setRelationship(baseRelationship);
            
            return edge;
        }, startLabel, endLabel, relationship);
    }

    private BaseEntity buildBaseEntity(Map<String, Object> nodeMap) {
        BaseEntity entity = new BaseEntity();
        entity.setId((Long) nodeMap.get("id"));
        entity.setName((String) nodeMap.get("name"));
        
        ArrayList<String> labels = new ArrayList<>();
        labels.add((String) nodeMap.get("category"));
        entity.setLabel(labels);
        
        Map<String, Object> properties = new HashMap<>((Map<String, Object>) nodeMap.get("properties"));
        properties.remove("name");
        entity.setProperties(properties);
        
        return entity;
    }
}