package com.qizhen.healsphere.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.qizhen.healsphere.service.KgService;
import com.qizhen.healsphere.web.dto.BaseNodeRSDTO;
import com.qizhen.healsphere.web.dto.GraphDTO;
import com.qizhen.healsphere.web.dto.NextNodeDTO;
import com.qizhen.healsphere.web.param.KgQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * PostgreSQL实现版本的知识图谱服务
 *
 * @author trae
 */
@Service("kgServicePostgres")
public class KgServicePostgresImpl implements KgService {

    @Autowired
    private JdbcTemplate jdbcTemplate;

    private Map<String, Object> getNodeProperties(Long nodeId) {
        String propSql = "SELECT prop_title, prop_value FROM kg_props WHERE ref_id = ?";
        List<Map<String, Object>> propResults = jdbcTemplate.query(
            propSql,
            (rs, rowNum) -> {
                Map<String, Object> prop = new HashMap<>();
                prop.put(rs.getString("prop_title"), rs.getString("prop_value"));
                return prop;
            },
            nodeId
        );
        
        Map<String, Object> properties = new HashMap<>();
        propResults.forEach(prop -> properties.putAll(prop));
        return properties;
    }

    @Override
    public List<GraphDTO> getGraph(KgQuery kgQuery) {
        List<GraphDTO> graphDTOList = new ArrayList<>();
        Map<String, GraphDTO> graphMap = new HashMap<>();
        Map<String, List<NextNodeDTO>> rtypeMap = new HashMap<>();

        // 1. 查询中心节点
        String nodeSql = "SELECT n.id, n.name, n.category " +
                "FROM kg_nodes n " +
                "WHERE n.category = ? AND n.name = ? " +
                "AND n.status = '0' ";

        jdbcTemplate.query(nodeSql, new Object[]{kgQuery.getLabelName(), kgQuery.getInputStr()},
                (rs, rowNum) -> {
                    String sName = rs.getString("name");
                    String sLabel = rs.getString("category");
                    long sId = rs.getLong("id");
                    Map<String, Object> sProp = getNodeProperties(sId);
                    sProp.put("name", sName);

                    GraphDTO graphDTO = new GraphDTO();
                    graphDTO.setName(sName);
                    graphDTO.setLabel(sLabel);
                    graphDTO.setId(sId);
                    graphDTO.setProperties(sProp);
                    graphMap.put(sName + "_" + sLabel, graphDTO);
                    return null;
                });

        // 2. 查询关联的边和目标节点
        String relationSql = "SELECT e.name as rType, m.id as target_id, m.name as target_name, " +
                "m.category as target_label, " +
                "(SELECT COUNT(*) FROM kg_edges WHERE src_id = m.id) as pCount " +
                "FROM kg_nodes n " +
                "JOIN kg_edges e ON n.id = e.src_id " +
                "JOIN kg_nodes m ON e.dest_id = m.id " +
                "WHERE n.category = ? AND n.name = ? " +
                "AND n.status = '0' " +
                "ORDER BY e.name";

        jdbcTemplate.query(relationSql, new Object[]{kgQuery.getLabelName(), kgQuery.getInputStr()},
                (rs, rowNum) -> {
                    String rType = rs.getString("rType");
                    String eName = rs.getString("target_name");
                    String eLabel = rs.getString("target_label");
                    long eId = rs.getLong("target_id");
                    Map<String, Object> eProp = getNodeProperties(eId);
                    eProp.put("name", eName);
                    int pCount = rs.getInt("pCount");

                    NextNodeDTO nextNodeDTO = new NextNodeDTO();
                    nextNodeDTO.setName(eName);
                    nextNodeDTO.setLabel(eLabel);
                    nextNodeDTO.setPCount(pCount);
                    nextNodeDTO.setId(eId);
                    nextNodeDTO.setProperties(eProp);

                    List<NextNodeDTO> nextNodeDTOS = rtypeMap.computeIfAbsent(rType, k -> new ArrayList<>());
                    nextNodeDTOS.add(nextNodeDTO);
                    return null;
                });

        // 3. 组装返回结果
        if (CollUtil.isNotEmpty(graphMap)) {
            GraphDTO graphDTO = graphMap.values().iterator().next();
            List<BaseNodeRSDTO> baseNodeRSDTOS = new ArrayList<>();

            rtypeMap.forEach((key, value) -> {
                BaseNodeRSDTO nodeRSDTO = new BaseNodeRSDTO();
                nodeRSDTO.setRType(key);
                nodeRSDTO.setENodeDTOS(value);
                baseNodeRSDTOS.add(nodeRSDTO);
            });

            graphDTO.setENodeRSDTOS(baseNodeRSDTOS);
            graphDTOList.add(graphDTO);
        }

        return graphDTOList;
    }

    @Override
    public List<Map<String, Object>> getGraphNew(KgQuery kgQuery) {
        String sql = "SELECT n.id as source_id, n.label as sLabel, n.name as sName, e.type as rType, " +
                "m.id as target_id, m.label as eLabel, m.name as eName, " +
                "(SELECT COUNT(*) FROM kg_edges WHERE source_id = m.id) as pCount " +
                "FROM kg_nodes n " +
                "JOIN kg_edges e ON n.id = e.source_id " +
                "JOIN kg_nodes m ON e.target_id = m.id " +
                "WHERE n.label = ? AND n.name = ? " +
                "AND (n.is_deleted = 'N' OR n.is_deleted IS NULL) " +
                "AND m.label IN ('疾病', '症状', '药品通用名', '手术和操作', '实验室检查', '辅助检查') " +
                "ORDER BY e.type";

        List<Map<String, Object>> results = jdbcTemplate.queryForList(sql, kgQuery.getLabelName(), kgQuery.getInputStr());
        
        // 为每个节点添加属性信息
        for (Map<String, Object> result : results) {
            Long sourceId = ((Number) result.get("source_id")).longValue();
            Long targetId = ((Number) result.get("target_id")).longValue();
            
            Map<String, Object> sourceProps = getNodeProperties(sourceId);
            Map<String, Object> targetProps = getNodeProperties(targetId);
            
            result.put("sProps", sourceProps);
            result.put("eProps", targetProps);
        }

        return results;
    }

    /**
     * 解析JSON格式的属性字符串为Map
     */
    private Map<String, Object> parseProperties(String propertiesJson) {
        // 这里需要根据实际存储格式实现属性的解析
        // 可以使用Jackson或Gson等JSON库进行解析
        return new HashMap<>(); // 临时返回空Map，需要实现具体的解析逻辑
    }
}